--------------------------------------------------------------------------------
-- DelayError: Counts delay errors
-- SubModule DelayError
-- CLK: clock
-- RST: reset
-- CNT: internal cycle count
-- ONEPPSADV: nominal 1 PPS of return signal
-- LOS: Loss of signal at the receiver
-- DLYERR: measered delay error
-- DLYOOR: high if fixed delay is out of range
-- DLYVLD: is high for 1 cycle when a new delay was measured
-- DELAYERR: high if a measured delay is not in the allowed window
-- MISSING: high if the 1PPS was missing on the return signal
-- UP: is high when the timing link is working properly
-- ERRCNT: this is count of errors (overflows)
-- Created   7/18/2008
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;
--synopsys translate_off
--library UNISIM;
--use unisim.vcomponents.all;
--synopsys translate_on

entity DelayError is port
   (
   CLK               : in  std_logic;
   RST               : in  std_logic;
   CNT               : in  std_logic_vector (25 downto 0);
   ONEPPSADV         : in  std_logic;
   LOS               : in  std_logic;
   DLYERR            : in  std_logic_vector (31 downto 0);
   DLYOOR            : in  std_logic;
   DLYVLD            : in  std_logic;
   DELAYERR          : out std_logic;
   MISSING           : out std_logic;
   UP                : out std_logic;
   ERRCNT            : out std_logic_vector (7 downto 0)
   );
end entity DelayError;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of DelayError is

-- Signal Declarations
   signal pps_sr       : std_logic_vector (7 downto 0);
   signal eval         : std_logic;
   signal dlyovf       : std_logic;
   signal timeout_cnt  : unsigned (17 downto 0);
   signal missflag     : std_logic;
   signal errflag      : std_logic;
   signal dflag        : std_logic;
   signal ecnt         : unsigned (7 downto 0);

begin

   -- generate clock enable for status evaluation at 2^16th cycle
   eval <= '1' when (Unsigned (CNT) = To_Unsigned (65536, 26)) else '0';

   -- delay overflow
   dlyovf <= '0' when (DLYERR(31 downto 8) = X"000000") or
                      (DLYERR(31 downto 8) = X"FFFFFF")
             else '1';

   -- timeout for DLYVLD
   timeout_proc: process (CLK) is
   begin
      if rising_edge (CLK) then
         if DLYVLD = '1' then
            timeout_cnt <= (others => '0');
         elsif (timeout_cnt /= X"FFFF"&B"1") then
            timeout_cnt <= timeout_cnt + 1;
         end if;
      end if;
   end process timeout_proc;
   -- was there a DLYVLD in the past 2^17 cycles?
   missflag <= '1' when (timeout_cnt = X"FFFF"&B"1") else '0';

   -- do we have an error condition?
   errflag <= dlyovf or DLYOOR or LOS or missflag;
   dflag <= (dlyovf or DLYOOR) and (not LOS) and (not missflag);
   err_count: process (CLK) is
   begin
      if rising_edge (CLK) then
         if EVAL = '1' then
            UP <= not errflag;
            DELAYERR <= dflag;
            MISSING <= missflag;
            if errflag = '1' then
               ecnt <= ecnt + 1;
            end if;
         end if;
      end if;
   end process err_count;
   ERRCNT <= std_logic_vector (ecnt);

end architecture Structure;


