--------------------------------------------------------------------------------
-- FanoutPortCounter: Adjust the counts for a fanout port
-- SubModule FanoutPortCounter
-- CLK: clock
-- RST: reset
-- SYNCDIS: Disables the synchronization function and keeps the old delay
-- DELAY: measured delay between outgoing and returned 1PPS
-- OOR: goes high if the delay is out of range
-- INCOUNT: clock count associated with internal VCO/1PPS
-- OUTCOUNT: clock count which is used to generate outgoing SYNC signal
-- NMI: not-minus of outgoing signal (low for 8 cycles)
-- ONEPPSADV: nominal 1 PPS of return signal (8 cycles long)
-- Created   7/18/2008
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;
--synopsys translate_off
--library UNISIM;
--use unisim.vcomponents.all;
--synopsys translate_on

entity FanoutPortCounter is port
   (
   CLK               : in  std_logic;
   RST               : in  std_logic;
   SYNCDIS           : in  std_logic;
   DELAY             : in  std_logic_vector (31 downto 0);
   OOR               : out std_logic;
   INCOUNT           : in  std_logic_vector (25 downto 0);
   OUTCOUNT          : out std_logic_vector (25 downto 0);
   NMI               : out std_logic;
   ONEPPSADV         : out std_logic
   );
end entity FanoutPortCounter;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of FanoutPortCounter is

-- Const Declarations
   constant delay_ofs : integer := 6;

-- Type Declarations

-- Signal Declarations
   signal delaycount       : unsigned (26 downto 0);
   signal clock            : unsigned (25 downto 0);
   signal count            : unsigned (25 downto 0);
   signal markpps          : unsigned (25 downto 0);
   signal advpps           : std_logic_vector (7 downto 0);

begin
   -- reference clock
   clocking: process (CLK, SYNCDIS, RST) is
   begin
      if rising_edge (CLK) then
         if (SYNCDIS = '0') or (RST = '1') then
            clock <= unsigned (INCOUNT);
         else
            clock <= clock + 1;
         end if;
      end if;
   end process clocking;

   -- adjust delay count
   delay_adjust: process (CLK, SYNCDIS) is
   begin
      if rising_edge (CLK) then
         if SYNCDIS = '0' then
            delaycount <=
               Unsigned (DELAY (31 downto 5)) + To_unsigned (delay_ofs, 27);
         end if;
      end if;
   end process delay_adjust;
   OOR <= '0' when (delaycount(26 downto 18) = B"000000000") else '1';

   -- generate local counter
   local_counter: process (CLK) is
   begin
      if rising_edge (CLK) then
         count <= unsigned (clock) + (B"0000000000" & delaycount(17 downto 2));
      end if;
   end process local_counter;
   OUTCOUNT <= std_logic_vector (count);
   NMI <= '0' when count (25 downto 3) = B"11111111111111111111111" else '1';

   -- generate advanced NMI (1 cylce long only)
   avd_nmi: process (CLK) is
   begin
      if rising_edge (CLK) then
         if (count (25 downto 0) = B"11111111111111111111111111") then
            markpps <= not count;
         end if;
      end if;
   end process;
   advpps(0) <= '1' when (markpps = count) else '0';
   advpps(7 downto 1) <= advpps (6 downto 0);
   -- generate advanced 1PPS
   ONEPPSADV <= advpps(7) or advpps(6) or advpps(5) or advpps(4) or
                advpps(3) or advpps(2) or advpps(1) or advpps(0);

end architecture Structure;

