#include <TROOT.h>
#include <TApplication.h>
#include <TSystem.h>
#include <TEnv.h>
#include <TTimer.h>
#include "Plot.hh"
#include "PlotSet.hh"
#include "FilterDesign.hh"
#include "Sine.hh"
#include "UniformNoise.hh"
#include "TSeries.hh"
#include "FSpectrum.hh"
#include "Histogram1.hh"
#include "Hanning.hh"
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>

using namespace ligogui;
using namespace std;


const double fS = 524288.;
const double dT = 0.5;
const bool writefiles = true;


TROOT root("GUI", "StimGen");

class CheckExit : public TTimer {
public:
   CheckExit (TApplication& app) 
   : TTimer (1000), fApp (&app) {
      Start (1000, kFALSE);
   }
   virtual Bool_t Notify() {
      if (gPlotSet().GetRegisteredWindows()->empty()) {
         fApp->Terminate();
      }
      return kTRUE;
   }
protected:
   TApplication*	fApp;
};


/* 
 * Read a response file
 */
bool readResp (const char* filename, int max,
               TSeries& ts1, TSeries& ts2)
{
   ifstream inp (filename);
   if (!inp) {
      cerr << "Error in opening file " << filename << endl;
      return false;
   }
   double* x1 = new double[max];
   double* x2 = new double[max];
   int len = 0;
   for (int i = 0; !inp.eof() && i < max; ++i, ++len) {
      inp >> x1[i] >> x2[i];
   }
   ts1 = TSeries (Time (0), Interval (1./fS), len, x1);
   ts2 = TSeries (Time (0), Interval (1./fS), len, x2);
   delete [] x1;
   delete [] x2;
   return inp;
}

/* 
 * Read a response file
 */
bool readResp (const char* filename, int max,
               TSeries& ts1)
{
   ifstream inp (filename);
   if (!inp) {
      cerr << "Error in opening file " << filename << endl;
      return false;
   }
   double* x1 = new double[max];
   int len = 0;
   for (int i = 0; !inp.eof() && i < max; ++i, ++len) {
      inp >> x1[i];
   }
   ts1 = TSeries (Time (0), Interval (1./fS), len, x1);
   delete [] x1;
   return inp;
}


/* 
 * Computes the power spectrum
 */
FSpectrum PSD (const TSeries& ts)
{
   Hanning win (ts.getNSample());
   TSeries out = win.apply (ts);
   FSpectrum ps (out);
   return ps;
}

int main(int argc, char **argv)
{
   TApplication theApp ("Bode Plot Test", &argc, argv);
   
   TSeries resp1, resp1i, resp1r;
   readResp ("../simdata_rtl/resp_1kHz_30b_1g_48a_05s.txt", fS*dT, resp1i, resp1);
   readResp ("../simdata_rtl/resp_rtl_1kHz_30b_1g_48a_05s.txt", fS*dT, resp1r);
   cout << "read resp_rtl_1kHz_30b_1g_48a_05s" << endl;
   
   resp1.setName ("fixed point (behaviour)");
   resp1i.setName ("floating point (ideal)");
   resp1r.setName ("fixed point (RTL)");
   Plot (resp1r, resp1, resp1i);
   
   double resp1rms  = sqrt (resp1*resp1/resp1.getNSample());
   double resp1irms = sqrt (resp1i*resp1i/resp1i.getNSample());
   double resp1rrms = sqrt (resp1r*resp1r/resp1r.getNSample());
   
   cout << "RMS behaviour = " << resp1rms << "  NSample = "
      <<resp1.getNSample() << endl;
   cout << "RMS ideal     = " << resp1irms << "  NSample = "
      <<resp1i.getNSample() << endl;
   cout << "RMS RTL       = " << resp1rrms << "  NSample = "
      <<resp1r.getNSample() << endl;
   
   resp1 *= (1./resp1rms);
   resp1i *= (1./resp1irms);
   resp1r *= (1./resp1rrms);
   TSeries diff = resp1 - resp1i;
   TSeries diffr = resp1r - resp1i;
   TSeries diffrr = resp1r - resp1;
   diff.setName ("Difference (behaviour/ideal)");
   diffr.setName ("Difference (RTL/ideal)");
   diffrr.setName ("Difference (RTL/behaviour)");
   Plot (diff, diffr, diffrr);
   
   Histogram1 hist1  ("Difference (behaviour/ideal)", 100, -4.0E-6, +2.0E-6);
   hist1.FillN (diff.getNSample(), (double*)diff.refData());
   Histogram1 hist1r ("Difference (RTL/ideal)",       100, -4.0E-6, +2.0E-6);
   hist1r.FillN (diffr.getNSample(), (double*)diffr.refData());
   Histogram1 hist1rr("Difference (RTL/behaviour)",   100, -4.0E-6, +2.0E-6);
   hist1rr.FillN (diffrr.getNSample(), (double*)diffrr.refData());
   Plot (hist1, hist1r, hist1rr);
   
//    TSeries resp100, resp100i;
//    readResp ("../simdata/resp_100Hz_16s.txt", fS*dT, resp100i, resp100);
//    cout << "read resp_100Hz_16s" << endl;
//    TSeries resp1000, resp1000i;
//    readResp ("../simdata/resp_1kHz_16s.txt", fS*dT, resp1000i, resp1000);
//    cout << "read resp_1kHz_16s" << endl;
//    TSeries resp10000, resp10000i;
//    readResp ("../simdata/resp_10kHz_16s.txt", fS*dT, resp10000i, resp10000);
//    cout << "read resp_10kHz_16s" << endl;
//    TSeries resp100000, resp100000i;
//    readResp ("../simdata/resp_100kHz_16s.txt", fS*dT, resp100000i, resp100000);
//    cout << "read resp_100kHz_16s" << endl;
//    TSeries twotone, twotonei;
//    readResp ("../simdata/resp_2tone_10Hz_1000Hz_16s.txt", fS*dT, twotonei, twotone);
//    cout << "read resp_2tone_10Hz_1000Hz_16s" << endl;
//    TSeries noise, noisei;
//    readResp ("../simdata/resp_noise_bs100_bs4k_16s.txt", fS*dT, noisei, noise);
//    cout << "read resp_noise_bs100_bs4k_16s" << endl;
//    TSeries noiselow, noiselowi;
//    readResp ("../simdata/resp_noiselow_bs100_bs4k_16s.txt", fS*dT, noiselowi, noiselow);
//    cout << "read resp_noiselow_bs100_bs4k_16s" << endl;
//    TSeries noiselowbit, noiselowbiti;
//    readResp ("../simdata/resp_noiselowbit_bs100_bs4k_16s.txt", fS*dT, noiselowbiti, noiselowbit);
//    cout << "read resp_noiselowbit_bs100_bs4k_16s" << endl;
  
//    FSpectrum ps0 = PSD(resp10);
//    FSpectrum ps1 = PSD(resp10i);
//    ps0.setName ("ELP6/30 bits/1g/48a");
//    ps1.setName ("ideal ELP6/30 bits/1g/48a");
//    Plot (ps0, ps1);
 //   FSpectrum ps2 = PSD(resp100);
//    FSpectrum ps3 = PSD(resp100i);
//    ps2.setName ("100Hz sine wave");
//    ps3.setName ("ideal 100Hz sine wave");
//    Plot (ps2, ps3);
//    FSpectrum ps4 = PSD(resp1000);
//    FSpectrum ps5 = PSD(resp1000i);
//    ps4.setName ("1kHz sine wave");
//    ps5.setName ("ideal 1kHz sine wave");
//    Plot (ps4, ps5);
//    FSpectrum ps6 = PSD(resp10000);
//    FSpectrum ps7 = PSD(resp10000i);
//    ps6.setName ("10kHz sine wave");
//    ps7.setName ("ideal 10kHz sine wave");
//    Plot (ps6, ps7);
//    FSpectrum ps8 = PSD(resp100000);
//    FSpectrum ps9 = PSD(resp100000i);
//    ps8.setName ("100kHz sine wave");
//    ps9.setName ("ideal 100kHz sine wave");
//    Plot (ps8, ps9);
//    FSpectrum ps10 = PSD(twotone);
//    FSpectrum ps11 = PSD(twotonei);
//    ps10.setName ("10Hz/1kHz two tone");
//    ps11.setName ("ideal 10Hz/1kHz two tone");
//    Plot (ps10, ps11);
//    FSpectrum ps12 = PSD(noise);
//    FSpectrum ps13 = PSD(noisei);
//    ps12.setName ("noise with band stop");
//    ps13.setName ("ideal noise with band stop");
//    Plot (ps12, ps13);
//    FSpectrum ps14 = PSD(noiselow);
//    FSpectrum ps15 = PSD(noiselowi);
//    ps14.setName ("low frequency noise");
//    ps15.setName ("ideal low frequency noise");
//    Plot (ps14, ps15);
//    FSpectrum ps16 = PSD(noiselowbit);
//    FSpectrum ps17 = PSD(noiselowbiti);
//    ps16.setName ("low frequency noise");
//    ps17.setName ("ideal low frequency noise");
//    Plot (ps16, ps17);
//    
//    Plot (ps0, ps2, ps4, ps6, ps8, ps10, ps12, ps16);

   CheckExit checkexit (theApp); 
   theApp.Run();
   
   return 0;
 }
