#include <TROOT.h>
#include <TApplication.h>
#include <TSystem.h>
#include <TEnv.h>
#include <TTimer.h>
#include "Plot.hh"
#include "PlotSet.hh"
#include "FilterDesign.hh"
#include "Sine.hh"
#include "UniformNoise.hh"
#include "TSeries.hh"
#include "FSpectrum.hh"
#include "Hanning.hh"
#include <math.h>
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>

using namespace ligogui;
using namespace std;


const double PI = 3.14159265358979323846;
const double fS = 524288.;
const double dT = 17.0;
const bool writefiles = true;


TROOT root("GUI", "StimGen");

class CheckExit : public TTimer {
public:
   CheckExit (TApplication& app) 
   : TTimer (1000), fApp (&app) {
      Start (1000, kFALSE);
   }
   virtual Bool_t Notify() {
      if (gPlotSet().GetRegisteredWindows()->empty()) {
         fApp->Terminate();
      }
      return kTRUE;
   }
protected:
   TApplication*	fApp;
};


/* 
 * Write a stimulus file
 */
bool writeStim (const char* filename, const TSeries& ts, bool inclX = false)
{
   ofstream out (filename);
   if (!out) return false;
   for (int i = 0; i < ts.getNSample(); ++i) {
      if (inclX) {
         out << setw(12) << right << 
            fixed << showpoint << setprecision (9) << 
            i*ts.getTStep() << " ";
      }
      out << setw(20) << right << 
         scientific << showpoint << setprecision (12) << 
         ts.getDouble (i) << endl;
   }
   return out;
}

/* 
 * Computes a sine wave
 */
TSeries sinewave (double f)
{
   Sine sss = Sine (f, 1.0);
   double* data = new double [fS * dT];
   
   Time t = 0.0;
   for (int i = 0; i < dT * fS; ++i) {
      data[i] = sss.Tspace (t);
      t += Interval(1./fS);
   }
   TSeries in = TSeries (Time (0), Interval (1./fS), dT*fS, data);
   delete [] data;
   return in;
}

/* 
 * Computes a square wave sweep
 */
TSeries squarewavesweep (double f1, double f2)
{
   double* data = new double [fS * dT];
   double f;
   double ph = 0.0;
   
   Time t = 0.0;
   for (int i = 0; i < dT * fS; ++i) {
      data[i] = (ph < 0.5) ? -1.0 : +1.0;
      f = f1 * exp ((double)i / (dT*fS) * log(f2/f1));
      ph += 1./fS * f;
      if (ph > 1.0) ph -= 1.0;
      t += Interval(1./fS);
   }
   TSeries in = TSeries (Time (0), Interval (1./fS), dT*fS, data);
   delete [] data;
   return in;
}

/* 
 * Computes uniform noise
 */
TSeries uniformnoise (int pad = 0)
{
   UniformNoise sss = UniformNoise (fS/2.);
   double* data = new double [fS * dT + pad];
   
   Time t = 0.0;
   for (int i = 0; i < dT * fS + pad; ++i) {
      data[i] = sss.Tspace (t);
      t += Interval(1./fS);
   }
   TSeries in = TSeries (Time (0), Interval (1./fS), dT*fS+pad, data);
   delete [] data;
   return in;
}

/* 
 * Computes the power spectrum
 */
FSpectrum PSD (const TSeries& ts)
{
   Hanning win (ts.getNSample());
   TSeries out = win.apply (ts);
   FSpectrum ps (out);
   return ps;
}

int main(int argc, char **argv)
{
   TApplication theApp ("Bode Plot Test", &argc, argv);

   // bit noise at high freq
   cout << "Generating bitnoise" << endl;
   TSeries bitnoise = uniformnoise (fS);
   FilterDesign bds (fS, "bit noise");
   bds.butter (kHighPass, 4, 200.E3);
   bitnoise = bds().apply (bitnoise);
   bitnoise.eraseStart (1.0);
   bitnoise = TSeries (Time (0), Interval (1./fS), *bitnoise.refDVect());
   bitnoise *= 54.E-6;
   cout << "  RMS = " << setw(10) << sqrt(bitnoise*bitnoise/bitnoise.getNSample()) <<
        "   Maximum " << setw(10) << bitnoise.getMaximum() << 
        "   Minimum " << setw(10) << bitnoise.getMinimum() << endl;

   cout << "Generating square wave sweep" << endl;
   TSeries stimSQR = squarewavesweep(1000, 10000);
   stimSQR *= 1.99;
   stimSQR += bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(stimSQR*stimSQR/stimSQR.getNSample()) <<
        "   Maximum " << setw(10) << stimSQR.getMaximum() << 
        "   Minimum " << setw(10) << stimSQR.getMinimum() << endl;
   if (writefiles) writeStim ("stim_sqrsweep_17s.txt", stimSQR);
//   Plot (stimSQR);
      
   cout << "Generating 10 Hz sine wave" << endl;
   TSeries stim10 = sinewave (10.0);
   stim10 += bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(stim10*stim10/stim10.getNSample()) <<
        "   Maximum " << setw(10) << stim10.getMaximum() << 
        "   Minimum " << setw(10) << stim10.getMinimum() << endl;
   if (writefiles) writeStim ("stim_10Hz_17s.txt", stim10);
   cout << "Generating 100 Hz sine wave" << endl;
   TSeries stim100 = sinewave (100.0);
   stim100 += bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(stim100*stim100/stim100.getNSample()) <<
        "   Maximum " << setw(10) << stim100.getMaximum() << 
        "   Minimum " << setw(10) << stim100.getMinimum() << endl;
   if (writefiles) writeStim ("stim_100Hz_17s.txt", stim100);
   cout << "Generating 1 kHz sine wave" << endl;
   TSeries stim1000 = sinewave (1000.0);
   stim1000 += bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(stim1000*stim1000/stim1000.getNSample()) <<
        "   Maximum " << setw(10) << stim1000.getMaximum() << 
        "   Minimum " << setw(10) << stim1000.getMinimum() << endl;
   if (writefiles) writeStim ("stim_1kHz_17s.txt", stim1000);
   cout << "Generating 10 kHz sine wave" << endl;
   TSeries stim10000 = sinewave (10000.0);
   stim10000 += bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(stim10000*stim10000/stim10000.getNSample()) <<
        "   Maximum " << setw(10) << stim10000.getMaximum() << 
        "   Minimum " << setw(10) << stim10000.getMinimum() << endl;
   if (writefiles) writeStim ("stim_10kHz_17s.txt", stim10000);
   cout << "Generating 100 kHz sine wave" << endl;
   TSeries stim100000 = sinewave (100000.0);
   stim100000 += bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(stim100000*stim100000/stim100000.getNSample()) <<
        "   Maximum " << setw(10) << stim100000.getMaximum() << 
        "   Minimum " << setw(10) << stim100000.getMinimum() << endl;
   if (writefiles) writeStim ("stim_100kHz_17s.txt", stim100000);
  
   cout << "Generating two tone" << endl;
   TSeries twotone = stim1000 + stim10;
   twotone *= 0.5;
   twotone += bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(twotone*twotone/twotone.getNSample()) <<
        "   Maximum " << setw(10) << twotone.getMaximum() << 
        "   Minimum " << setw(10) << twotone.getMinimum() << endl;
   if (writefiles) writeStim ("stim_2tone_10Hz_1000Hz_17s.txt", twotone);
   
   cout << "Generating uniform noise" << endl;
   TSeries stimnoise = uniformnoise(fS);
   FilterDesign ds (fS, "Stopband");
   ds.ellip (kBandStop, 8, 0.2, 80, 100, 200);
   ds.ellip (kBandStop, 8, 0.2, 80, 4000, 4200);
   ds.ellip (kBandStop, 8, 0.2, 80, 100, 200);
   ds.ellip (kBandStop, 8, 0.2, 80, 4000, 4200);
   TSeries stimnoisenotch = ds().apply (stimnoise);
   stimnoisenotch.eraseStart (1.0);
   stimnoisenotch = TSeries (Time (0), Interval (1./fS), *stimnoisenotch.refDVect());
   cout << "  RMS = " << setw(10) << sqrt(stimnoisenotch*stimnoisenotch/stimnoisenotch.getNSample()) <<
        "   Maximum " << setw(10) << stimnoisenotch.getMaximum() << 
        "   Minimum " << setw(10) << stimnoisenotch.getMinimum() << endl;
   if (writefiles) writeStim ("stim_noise_bs100_bs4k_17s.txt", stimnoisenotch);
   
   cout << "Generating low frequency noise" << endl;
   FilterDesign bl (fS, "Low pass");
   bl.filter ("zpk([1000;1000;1000;1000;1000;1000],[10;10;10;10;10;10],265.,'n')");
   TSeries stimnoiselow = ds().apply (bl().apply (stimnoise));
   stimnoiselow.eraseStart (1.0);
   stimnoiselow = TSeries (Time (0), Interval (1./fS), *stimnoiselow.refDVect());
   cout << "  RMS = " << setw(10) << sqrt(stimnoiselow*stimnoiselow/stimnoiselow.getNSample()) <<
        "   Maximum " << setw(10) << stimnoiselow.getMaximum() << 
        "   Minimum " << setw(10) << stimnoiselow.getMinimum() << endl;
   if (writefiles) writeStim ("stim_noiselow_bs100_bs4k_17s.txt", stimnoiselow);
   cout << "Generating low frequency noise with bit noise" << endl;
   TSeries stimnoiselowbit = stimnoiselow + bitnoise;
   cout << "  RMS = " << setw(10) << sqrt(stimnoiselowbit*stimnoiselowbit/stimnoiselowbit.getNSample()) <<
        "   Maximum " << setw(10) << stimnoiselowbit.getMaximum() << 
        "   Minimum " << setw(10) << stimnoiselowbit.getMinimum() << endl;
   if (writefiles) writeStim ("stim_noiselowbit_bs100_bs4k_17s.txt", stimnoiselowbit);
   
//    FSpectrum ps1 = PSD(stimnoiselow);
//    ps1.setName ("low frequency noise");
//    FSpectrum ps2 = PSD(stimnoiselowbit);
//    ps2.setName ("low frequency noise w/ bit noise");
//    FSpectrum ps3 = PSD(stimnoisenotch);
//    ps3.setName ("flat noise");
//    Plot (ps1, ps2, ps3);

   FSpectrum ps1 = PSD(stim10);
   ps1.setName ("10Hz sine wave");
   FSpectrum ps2 = PSD(stim100);
   ps2.setName ("100Hz sine wave");
   FSpectrum ps3 = PSD(stim1000);
   ps3.setName ("1kHz sine wave");
   FSpectrum ps4 = PSD(stim10000);
   ps4.setName ("10kHz sine wave");
   FSpectrum ps5 = PSD(stim100000);
   ps5.setName ("100kHz sine wave");
   FSpectrum ps6 = PSD(twotone);
   ps6.setName ("10Hz/1kHz two tone");
   FSpectrum ps7 = PSD(stimnoisenotch);
   ps7.setName ("noise with band stop");
   FSpectrum ps8 = PSD(stimnoiselowbit);
   ps8.setName ("low frequency noise");
   Plot (ps1, ps2, ps3, ps4, ps5, ps6, ps7, ps8);
   
   
   CheckExit checkexit (theApp); 
   theApp.Run();
   
   return 0;
 }
